require('pegjs-require');
var log4js = require('log4js');
var parser = require('./grammer/javascript.pegjs');

var logger = log4js.getLogger();

// internal use
// extract common function params from command body
function extractFuncParams(cmdBody) {
    var funcData = {};
    funcData.name = cmdBody.callee.name;
    var arr = [];
    // iterate each argument
    for (var i in cmdBody.arguments) {
        var argument = cmdBody.arguments[i];
        if (argument.type === 'BinaryExpression') {
            arr.push(argument.left.value);
            arr.push(argument.right.value);
        } else if (argument.type === 'Literal') {
            arr.push(argument.value);
        } else if (argument.type === 'MemberExpression') {
            arr.push(argument.object.name + '.' + argument.property.name);
        } else if (argument.type === 'Identifier') {
            arr.push(argument.name);
        }
    }
    funcData.params = arr;
    return funcData;
}

function parse_switch(str)
{
    var cmd = parser.parse(str);

    // validate 
    if(cmd.type !== 'Program' || cmd.body[0] === 'undefined' )
        return undefined;

    // validate switch
    if(cmd.body[0].type !== 'SwitchStatement')
        return undefined;

    var switchName = cmd.body[0].discriminant.name;

    var casesArray = cmd.body[0].cases;
    var cases = [];
    for (var  i = 0; i < casesArray.length; i++) {
        var objCase = {};

        // test
        if (casesArray[i].test ) {
    
            if( casesArray[i].test.type === 'Identifier')   
                objCase.test = casesArray[i].test.name;
            else    
                objCase.test = casesArray[i].test.value;            
        }
        else {
            objCase.test = '$DEFAULT';
        }

        // consequent
        if( casesArray[i].consequent[0].expression.left.type === 'Identifier')  
            objCase.consequentLeft = casesArray[i].consequent[0].expression.left.name;
        else    
            objCase.consequentLeft = casesArray[i].consequent[0].expression.left.value;

        if( casesArray[i].consequent[0].expression.right.type === 'Identifier')  
            objCase.consequentRight = casesArray[i].consequent[0].expression.right.name;
        else    
            objCase.consequentRight = casesArray[i].consequent[0].expression.right.value;    
 
        cases.push(objCase);
    }

    var returnObj = {};
    returnObj.funcName = 'switch';
    returnObj.params = {};
    returnObj.params.discriminant = switchName; 
    returnObj.params.cases = cases; 

    return returnObj;
}

function parse_ifBinary(str) 
{
    //console.log("str in ifbinary: " + str);
    var cmd = parser.parse(str);
    //console.log("ifbinary cmd: " + JSON.stringify(cmd));

    // validate 
    if(cmd.type !== 'Program' || cmd.body[0] === 'undefined' )
        return undefined;

    // validate
    var cmdBody = cmd.body[0];
    if(cmdBody.type !== 'IfStatement' || cmdBody.test === undefined || cmdBody.consequent === undefined || cmdBody.alternate === undefined )
        return undefined;  

    // validate test (has to be binary)
    if( cmdBody.test.type !== 'BinaryExpression' || cmdBody.test.operator === undefined || 
        cmdBody.test.left === undefined || cmdBody.test.right === undefined )
        return undefined;

    // validate consequent
    if( cmdBody.consequent.type !== 'ExpressionStatement' || cmdBody.alternate.type != 'ExpressionStatement' )
        return undefined;
    
    var params = {};
    
    //console.log("cmdBody: " + JSON.stringify(cmdBody));

    if(cmdBody.test.left.type === 'MemberExpression') {
        params.testLeft = cmdBody.test.left.object.name + '.' + cmdBody.test.left.property.name;
    } else
        params.testLeft = cmdBody.test.left.name;
    
    //console.log("params.testLeft: " + JSON.stringify(params.testLeft));
    params.testOperator = cmdBody.test.operator;

    if( cmdBody.test.right.type === 'Identifier')
        params.testRight = cmdBody.test.right.name;
    else   
        params.testRight = cmdBody.test.right.value;


    params.consequentLeft = cmdBody.consequent.expression.left.name;
    
    if( cmdBody.consequent.expression.right.type === 'Identifier')
        params.consequentRight = cmdBody.consequent.expression.right.name;
    else {
        if( cmdBody.consequent.expression.right.type === 'CallExpression'){
            var cmdBody1 = cmdBody.consequent.expression.right;
            //console.log("cmdBody1: " + JSON.stringify(cmdBody1));
            var funcName = cmdBody1.callee.name;

            var data = {}; 
            data.name = funcName;
            var arr = [];
            // iterate each argument
            for (var i in cmdBody1.arguments) {
                var argument = cmdBody1.arguments[i];
                if (argument.type === 'BinaryExpression') {
                    arr.push(argument.left.value);
                    arr.push(argument.right.value); 
                } else if(argument.type === 'Literal') {
                    arr.push(argument.value);
                } else if(argument.type === 'MemberExpression'){
                    arr.push(argument.object.name + '.' + argument.property.name);
                }
            }
            
            data.params = arr;
            //console.log("data: " + JSON.stringify(data));
            params.consequentRight = data;
        }
        else
            params.consequentRight = cmdBody.consequent.expression.right.value;
    }   
    
    params.alternateLeft = cmdBody.alternate.expression.left.name;
    
    if( cmdBody.alternate.expression.right.type === 'Identifier')
        params.alternateRight = cmdBody.alternate.expression.right.name;
    else{
        if( cmdBody.alternate.expression.right.type === 'CallExpression'){
            var cmdBody1 = cmdBody.alternate.expression.right;
            //console.log("cmdBody1: " + JSON.stringify(cmdBody1));
            var funcName = cmdBody1.callee.name;

            var data = {}; 
            data.name = funcName;
            var arr = [];
            // iterate each argument
            for (var i in cmdBody1.arguments) {
                var argument = cmdBody1.arguments[i];
                if (argument.type === 'BinaryExpression') {
                    arr.push(argument.left.value);
                    arr.push(argument.right.value); 
                } else if(argument.type === 'Literal') {
                    arr.push(argument.value);
                } else if(argument.type === 'MemberExpression'){
                    arr.push(argument.object.name + '.' + argument.property.name);
                }
            }
            
            data.params = arr;
            //console.log("data: " + JSON.stringify(data));
            params.alternateRight = data;
        }
        else
            params.alternateRight = cmdBody.alternate.expression.right.value;
    }   
    
    
            

    var returnObj = {};
    returnObj.funcName = 'ifBinary'
    returnObj.params = params; 
    
    //console.log("returnObj: " + JSON.stringify(returnObj));
    
    return returnObj;

} // parse_ifBinary


// get the rule for if statement with a function as test condition
function parse_ifFunction(str) {
    var cmd = parser.parse(str);
    // console.log("if-function cmd: " + JSON.stringify(cmd));

    // validate basics
    if (cmd.type !== 'Program' || !cmd.body[0])
        return null;

    var cmdBody = cmd.body[0];
    // console.log("cmdBody: " + JSON.stringify(cmdBody));

    // function field validation
    if (cmdBody.type !== 'IfStatement' || !cmdBody.test || !cmdBody.consequent || !cmdBody.alternate)
        return null;

    // function type validation
    if (cmdBody.test.type !== 'CallExpression' || !cmdBody.test.callee || !cmdBody.test.arguments)
        return null;

    // consequent and alternative validation for expression statement
    if (cmdBody.consequent.type !== 'ExpressionStatement' || cmdBody.alternate.type != 'ExpressionStatement')
        return null;

    var params = {}; // define the params for the if function

    // 1. get the parameters for the test function    
    var testFun = extractFuncParams(cmdBody.test);;
    // console.log("test func: " + JSON.stringify(testFun));
    params.test = testFun;

    // 2. get name for left consequent (the left part for if evaluation)
    params.consequentLeft = cmdBody.consequent.expression.left.name;

    // 3. get the right expression for right consequent (the right part for if evaluation)
    if (cmdBody.consequent.expression.right.type === 'Identifier')
        params.consequentRight = cmdBody.consequent.expression.right.name;
    else {
        if (cmdBody.consequent.expression.right.type === 'CallExpression') {
            var funcData = extractFuncParams(cmdBody.consequent.expression.right);
            // console.log("funcData: " + JSON.stringify(funcData));
            params.consequentRight = funcData;
        } else
            params.consequentRight = cmdBody.consequent.expression.right.value;
    }

    // 4. get name for left alternate (the left part for else evaluation)
    params.alternateLeft = cmdBody.alternate.expression.left.name;

    // 5. get right expression for right alternate (the right part for else evaluation)
    if (cmdBody.alternate.expression.right.type === 'Identifier')
        params.alternateRight = cmdBody.alternate.expression.right.name;
    else {
        if (cmdBody.alternate.expression.right.type === 'CallExpression') {
            var funcData = extractFuncParams(cmdBody.alternate.expression.right);
            // console.log("funcData: " + JSON.stringify(funcData));
            params.alternateRight = funcData;
        } else
            params.alternateRight = cmdBody.alternate.expression.right.value;
    }

    // 6. pack it up and deliver
    var returnObj = {};
    returnObj.name = 'ifFunction'
    returnObj.params = params;
    //console.log("returnObj: " + JSON.stringify(returnObj));
    return returnObj;
} // parse_ifFunction

function parse_func(str)
{
    var cmd = parser.parse(str);
    // console.log("in Parse_func cmd: " + JSON.stringify(cmd));

    // validate 
    if (cmd.type !== 'Program' || !cmd.body[0])
        return null;

    // validate
    var cmdBody = cmd.body[0];
    if (cmdBody.type !== 'ExpressionStatement' || cmdBody.expression.type !== 'CallExpression')
        return null;

    // validate 
    if (cmdBody.expression.callee.type !== 'Identifier')
        return null;

    var funcName = cmdBody.expression.callee.name;
    var params = [];

    for (var i = 0; i < cmdBody.expression.arguments.length; i++) {
        var argument = cmdBody.expression.arguments[i];
        if (argument.type === 'Literal')
            params.push(argument.value);
        else if (argument.type === 'Identifier')
            params.push(argument.name);
    }

    var returnObj = {};
    returnObj.name = funcName;
    returnObj.params = params;

    // console.log("in Parse_func returnObj: " + JSON.stringify(returnObj));
    return returnObj;
} // parse_func



module.exports = {
    parse_switch,
    parse_ifBinary,
    parse_ifFunction,
    parse_func
};



